@@ -21,9 +21,11 @@ from collections import defaultdict
 from carbon.conf import settings
 from carbon import events, log
 from carbon.pipeline import Processor
+from carbon.util import TaggedSeries
 
 
-def by_timestamp((timestamp, value)):  # useful sort key function
+def by_timestamp(t_v):  # useful sort key function
+  (timestamp, _) = t_v
   return timestamp
 
 
@@ -31,10 +33,16 @@ class CacheFeedingProcessor(Processor):
   plugin_name = 'write'
 
   def __init__(self, *args, **kwargs):
-    super(Processor, self).__init__(*args, **kwargs)
+    super(CacheFeedingProcessor, self).__init__(*args, **kwargs)
     self.cache = MetricCache()
 
   def process(self, metric, datapoint):
+    # normalize metric name (reorder tags)
+    try:
+      metric = TaggedSeries.parse(metric).path
+    except Exception as err:
+      log.msg('Error parsing metric %s: %s' % (metric, err))
+
     self.cache.store(metric, datapoint)
     return Processor.NO_OUTPUT
 
@@ -47,7 +55,7 @@ class DrainStrategy(object):
     self.cache = cache
 
   def choose_item(self):
-    raise NotImplemented
+    raise NotImplementedError()
 
 
 class NaiveStrategy(DrainStrategy):
@@ -57,14 +65,14 @@ class NaiveStrategy(DrainStrategy):
 
     def _generate_queue():
       while True:
-        metric_names = self.cache.keys()
+        metric_names = list(self.cache.keys())
         while metric_names:
           yield metric_names.pop()
 
     self.queue = _generate_queue()
 
   def choose_item(self):
-    return self.queue.next()
+    return next(self.queue)
 
 
 class MaxStrategy(DrainStrategy):
@@ -73,14 +81,14 @@ class MaxStrategy(DrainStrategy):
   that infrequently or irregularly updated metrics may not be written
   until shutdown """
   def choose_item(self):
-    metric_name, size = max(self.cache.items(), key=lambda x: len(itemgetter(1)(x)))
+    metric_name, _ = max(self.cache.items(), key=lambda x: len(itemgetter(1)(x)))
     return metric_name
 
 
 class RandomStrategy(DrainStrategy):
   """Pop points randomly"""
   def choose_item(self):
-    return choice(self.cache.keys())
+    return choice(list(self.cache.keys()))  # nosec
 
 
 class SortedStrategy(DrainStrategy):
@@ -105,7 +113,7 @@ class SortedStrategy(DrainStrategy):
     self.queue = _generate_queue()
 
   def choose_item(self):
-    return self.queue.next()
+    return next(self.queue)
 
 
 class TimeSortedStrategy(DrainStrategy):
@@ -119,9 +127,14 @@ class TimeSortedStrategy(DrainStrategy):
       while True:
         t = time.time()
         metric_lw = sorted(self.cache.watermarks, key=lambda x: x[1], reverse=True)
+        if settings.MIN_TIMESTAMP_LAG:
+          metric_lw = [x for x in metric_lw if t - x[1] > settings.MIN_TIMESTAMP_LAG]
         size = len(metric_lw)
         if settings.LOG_CACHE_QUEUE_SORTS and size:
           log.msg("Sorted %d cache queues in %.6f seconds" % (size, time.time() - t))
+        if not metric_lw:
+          # If there is nothing to do give a chance to sleep to the reader.
+          yield None
         while metric_lw:
           yield itemgetter(0)(metric_lw.pop())
         if settings.LOG_CACHE_QUEUE_SORTS and size:
@@ -130,7 +143,7 @@ class TimeSortedStrategy(DrainStrategy):
     self.queue = _generate_queue()
 
   def choose_item(self):
-    return self.queue.next()
+    return next(self.queue)
 
 
 class _MetricCache(defaultdict):
@@ -174,7 +187,9 @@ class _MetricCache(defaultdict):
       metric = self.strategy.choose_item()
     else:
       # Avoid .keys() as it dumps the whole list
-      metric = self.iterkeys().next()
+      metric = next(iter(self))
+    if metric is None:
+      return (None, [])
     return (metric, self.pop(metric))
 
   def get_datapoints(self, metric):
@@ -207,6 +222,7 @@ class _MetricCache(defaultdict):
 
 _Cache = None
 
+
 def MetricCache():
   global _Cache
   if _Cache is not None:
@@ -230,6 +246,5 @@ def MetricCache():
   return _Cache
 
 
-
 # Avoid import circularities
-from carbon import state
+from carbon import state  # NOQA
