@@ -1,6 +1,3 @@
-from twisted.python.failure import Failure
-
-
 class Event:
   def __init__(self, name):
     self.name = name
@@ -19,7 +16,8 @@ class Event:
       try:
         handler(*args, **kwargs)
       except Exception:
-        log.err(None, "Exception in %s event handler: args=%s kwargs=%s" % (self.name, args, kwargs))
+        log.err(
+          None, "Exception in %s event handler: args=%s kwargs=%s" % (self.name, args, kwargs))
 
 
 metricReceived = Event('metricReceived')
@@ -29,7 +27,12 @@ cacheSpaceAvailable = Event('cacheSpaceAvailable')
 pauseReceivingMetrics = Event('pauseReceivingMetrics')
 resumeReceivingMetrics = Event('resumeReceivingMetrics')
 
-cacheFull.addHandler(lambda: carbon.instrumentation.increment('cache.overflow'))
+# Default handlers
+metricReceived.addHandler(
+  lambda metric, datapoint: state.instrumentation.increment('metricsReceived'))
+
+
+cacheFull.addHandler(lambda: state.instrumentation.increment('cache.overflow'))
 cacheFull.addHandler(lambda: setattr(state, 'cacheTooFull', True))
 cacheSpaceAvailable.addHandler(lambda: setattr(state, 'cacheTooFull', False))
 
@@ -38,4 +41,4 @@ resumeReceivingMetrics.addHandler(lambda: setattr(state, 'metricReceiversPaused'
 
 
 # Avoid import circularities
-from carbon import log, state
+from carbon import log, state  # NOQA
