@@ -5,4 +5,9 @@ several modules.
 
 metricReceiversPaused = False
 cacheTooFull = False
+client_manager = None
 connectedMetricReceiverProtocols = set()
+pipeline_processors = []
+pipeline_processors_generated = []
+database = None
+listeningPorts = []
